;********************************************************
;
; Stepper Motor controller
; Adapted from www.interq.or.jp/japan/se-inoue/e_step.htm
; Written by Seiichi Inoue
; Modified by Dan Simon
;
;********************************************************
        
	list            p=pic16f877
        include         p16f877.inc

;****************  Label Definition  ********************
        cblock  0x20
	mode                            ;Operation mode; 0=stop 1=right 2=left
	count1                          ;Wait counter
	count2                          ;Wait counter(for 1msec)
        endc

rb0     	equ     0               ;RB0 of PORTB
rb1     	equ     1               ;RB1 of PORTB
rb2     	equ     2               ;RB2 of PORTB
rb5    		equ     5               ;RB5 of PORTB
rb7     	equ     7               ;RB7 of PORTB
WaitTime	equ	d'50'
Position1	equ	b'0101'
Position2	equ	b'1001'
Position3	equ	b'1010'
Position4	equ	b'0110'

;****************  Program Start  ***********************
        org     0               ;Reset Vector
	nop			; Reserved for ICD
        goto    init
        org     4               ;Interrupt Vector
        clrf    INTCON          ;Clear Interrupt reg

;****************  Initial Process  *********************
init
        banksel	TRISA
	movlw	0xC0
        movwf	TRISA           ;Set PORTA all OUT
        movlw   b'00100111'     ;RB0,1,2,5=IN RB3,4,6,7=OUT
        movwf   TRISB           ;Set PORTB
        movlw   b'10000000'     ;RBPU=1 Pull up not use
        movwf   OPTION_REG      ;Set OPTION_REG
	movlw	b'00000110'	; Configure PORTA bits as digital
	movwf	ADCON1
        banksel	PORTA
        clrf    mode            ;Set mode = stop
        clrf    count1          ;Clear counter
        clrf    count2          ;Clear counter
	movlw	Position1
        movwf   PORTA           ;Write PORTA

; Speed control
;        bsf     PORTB,rb7       ;Set RB7 = 1
;        btfsc   PORTB,rb5       ;RB5 = 0 ?
;        goto    $-1             ;No. Wait

start
;*************  Check switch condition  *****************
;        btfsc   PORTB,rb1       ;RB1(stop key) = ON ?
;        goto    check1          ;No. Next
;        clrf    mode            ;Yes. Set stop mode
;        goto    drive           ;No. Jump to motor drive
;check1
;        btfsc   PORTB,rb2       ;RB2(right key) = ON ?
;        goto    check2          ;No. Next
;        movlw   d'1'            ;Yes. Set right mode
;        movwf   mode            ;Save mode
;        goto    drive           ;No. Jump to motor drive
;check2
;        btfsc   PORTB,rb0       ;RB0(left key) = ON ?
;        goto    drive           ;No. Jump to motor drive
        movlw   d'2'            ;Yes. Set left mode
        movwf   mode            ;Save mode

;********************  Motor drive  *********************
drive
        movf    mode,W          ;Read mode
        bz      start           ;mode = stop

; Speed control
;        bsf     PORTB,rb7       ;Set RB7 = 1
;        btfsc   PORTB,rb5       ;RB5 = 0 ?
;        goto    $-1             ;No. Wait

        movlw   WaitTime	;Set loop count (1 msec units)
        movwf   count1          ;Save loop count
loop    call    timer           ;Wait 1 msec
        decfsz  count1,f        ;count - 1 = 0 ?
        goto    loop            ;No. Continue

; Speed control
;        bcf     PORTB,rb7       ;Set RB7 = 0
;        btfss   PORTB,rb5       ;RB5 = 1 ?
;        goto    $-1             ;No. Wait

        movf    PORTA,w         ;Read PORTA
	sublw	Position1
        bnz     drive2          ;Unmatch
        movf    mode,w          ;Read mode
        sublw   d'1'            ;Right ?
        bz      drive1          ;Yes. Right
	movlw	Position2
        goto    drive_end       ;Jump to PORTA write
drive1
        movlw   Position4	;Set Right data
        goto    drive_end       ;Jump to PORTA write
;-------
drive2
        movf    PORTA,w         ;Read PORTA
	sublw	Position4
        bnz     drive4          ;Unmatch
        movf    mode,w          ;Read mode
        sublw   d'1'            ;Right ?
        bz      drive3          ;Yes. Right
	movlw	Position1
        goto    drive_end       ;Jump to PORTA write
drive3
        movlw   Position3	;Set Right data
        goto    drive_end       ;Jump to PORTA write
;-------
drive4
        movf    PORTA,w         ;Read PORTA
	sublw	Position3
        bnz     drive6          ;Unmatch
        movf    mode,W          ;Read mode
        sublw   d'1'            ;Right ?
        bz      drive5          ;Yes. Right
	movlw	Position4
        goto    drive_end       ;Jump to PORTA write
drive5
        movlw   Position2	;Set Right data
        goto    drive_end       ;Jump to PORTA write
;-------
drive6
        movf    PORTA,w         ;Read PORTA
	sublw	Position2	
        bnz     drive8          ;Unmatch
        movf    mode,w          ;Read mode
        sublw   d'1'            ;Right ?
        bz      drive7          ;Yes. Right
	movlw	Position3
        goto    drive_end       ;Jump to PORTA write
drive7
        movlw   Position1	;Set Right data
        goto    drive_end       ;Jump to PORTA write
;-------
drive8
	movlw	Position1
drive_end
        movwf   PORTA           ;Write PORTA
        goto    start           ;Jump to start

;*************  1 msec Delay Subroutine  *****************
; Assuming that the clock speed is 4 MHz
timer
        movlw   d'200'          ;Set loop count
        movwf   count2          ;Save loop count
timer1	nop                     ;Time adjust
        nop                     ;Time adjust
        decfsz  count2,f        ;count - 1 = 0 ?
        goto    timer1		;No. Continue
        return                  ;Yes. Count end

        end

